'''
To understand the index:
https://csis.myclimateservice.eu/node/3527
https://www.climdex.org/learn/indices/#index-TX10p
http://etccdi.pacificclimate.org/list_27_indices.shtml

'''

from .Ana import Ana
from .Inemet import Inemet
from .Ideam import Ideam
from .Merge import Merge
from .Alexandre import Alexandre

